use crate::token::TokenType::Slash;
use std::{iter::Peekable, str::CharIndices};

use crate::token::Token;

use super::interface::Tokenizer;

/// Consume comments.
///
/// A comment starts with '//' and runs until the end of the line.
/// If only one '/' is seen, it is consumed as a Slash token.
pub struct Comment;
impl Tokenizer for Comment {
    fn run(
        &self,
        c: (usize, char),
        chars: &mut Peekable<CharIndices<'_>>,
        source: &str,
        line: usize,
    ) -> Option<(usize, Option<Token>)> {
        match c.1 {
            '/' => {
                let (line_advance, token) = if chars.next_if(|(_, peek)| *peek == '/').is_some() {
                    while chars.next_if(|(_, peek)| *peek != '\n').is_some() {}
                    chars.next();
                    (1, None)
                } else {
                    (
                        0,
                        Some(Token {
                            token_type: Slash,
                            lexeme: source[c.0..=c.0].to_string(),
                            literal: None,
                            line,
                        }),
                    )
                };
                Some((line_advance, token))
            }
            _ => None,
        }
    }
}
