use crate::token::{Literal, Token, TokenType};
use ordered_float::OrderedFloat;
use std::{iter::Peekable, str::CharIndices};
use tracing::error;

use super::interface::Tokenizer;

/// Consume a number literal. Numbers can have one decimal point.
pub struct Number;
impl Tokenizer for Number {
    fn run(
        &self,
        c: (usize, char),
        chars: &mut Peekable<CharIndices<'_>>,
        source: &str,
        line: usize,
    ) -> Option<(usize, Option<Token>)> {
        if c.1.is_ascii_digit() {
            let mut end_idx = c.0;
            while let Some((idx, _)) = chars.next_if(|(_, x)| x.is_ascii_digit()) {
                end_idx = idx;
            }
            chars.next_if(|(_, x)| *x == '.');
            while let Some((idx, _)) = chars.next_if(|(_, x)| x.is_ascii_digit()) {
                end_idx = idx;
            }

            let lexeme = source[c.0..=end_idx].to_string();
            let token = match lexeme.parse::<f64>() {
                Ok(literal) => Some(Token {
                    token_type: TokenType::Number,
                    lexeme,
                    literal: Some(Literal::Number(OrderedFloat(literal))),
                    line,
                }),
                Err(e) => {
                    error!("failed to parse number: {e}");
                    None
                }
            };

            Some((0, token))
        } else {
            None
        }
    }
}
