use std::{fmt::Display, rc::Rc};

use ordered_float::OrderedFloat;

use crate::{callable::Callable, token::Literal};

/// Concrete value in the interpreter.
#[derive(Clone, Debug, PartialEq)]
pub enum Value {
    Return(Box<Value>),
    Callable(Rc<dyn Callable>),
    String(String),
    Number(OrderedFloat<f64>),
    Boolean(bool),
    Nil,
}

impl Display for Value {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Value::Return(x) => write!(f, "{x}"),
            Value::Callable(x) => write!(f, "{x}"),
            Value::String(x) => write!(f, "{x}"),
            Value::Number(x) => write!(f, "{x}"),
            Value::Boolean(x) => write!(f, "{x}"),
            Value::Nil => write!(f, "nil"),
        }
    }
}

impl Value {
    /// Return false for nil or a false boolean, true otherwise.
    pub fn is_truthy(&self) -> bool {
        match self {
            Value::Nil => false,
            Value::Boolean(b) => *b,
            _ => true,
        }
    }
}

impl From<Literal> for Value {
    fn from(value: Literal) -> Self {
        match value {
            Literal::String(x) => Value::String(x),
            Literal::Number(x) => Value::Number(x),
            Literal::Boolean(x) => Value::Boolean(x),
            Literal::Nil => Value::Nil,
        }
    }
}
